@@ -50,7 +50,7 @@ module Page =
 struct
   type t = string*string
 
-  external serialno : t -> int = "ocaml_ogg_page_serialno"
+  external serialno : t -> nativeint = "ocaml_ogg_page_serialno"
 
   external eos : t -> bool = "ocaml_ogg_page_eos"
 
@@ -62,9 +62,9 @@ struct
 
   external version : t -> int = "ocaml_ogg_page_version"
 
-  external granulepos : t -> int = "ocaml_ogg_page_granulepos"
+  external granulepos : t -> Int64.t = "ocaml_ogg_page_granulepos"
 
-  external pageno : t -> int = "ocaml_ogg_page_pageno"
+  external pageno : t -> nativeint = "ocaml_ogg_page_pageno"
 
   external set_checksum : t -> unit = "ocaml_ogg_page_checksum_set"
 
@@ -76,9 +76,13 @@ struct
 
   type packet
 
-  external create : int -> stream = "ocaml_ogg_stream_init"
+  external create : nativeint -> stream = "ocaml_ogg_stream_init"
 
-  let create ?(serial = Random.int 0x3FFFFFFF) () = create serial
+  let create ?(serial = Random.nativeint (Nativeint.of_int 0x3FFFFFFF)) () = create serial
+
+  external serialno : stream -> nativeint = "ocaml_ogg_stream_serialno"
+
+  external eos : stream -> bool = "ocaml_ogg_stream_eos"
 
   external get_page : stream -> Page.t = "ocaml_ogg_stream_pageout"
 
@@ -88,6 +92,10 @@ struct
     let callback () = sync := true in
     get_packet callback s
 
+  external peek_packet : stream -> packet = "ocaml_ogg_stream_packetpeek"
+
+  external put_packet : stream -> packet -> unit = "ocaml_ogg_stream_packetin"
+
   external put_page : stream -> Page.t -> unit = "ocaml_ogg_stream_pagein"
 
   external flush_page : stream -> Page.t = "ocaml_ogg_flush_stream"
@@ -151,7 +159,7 @@ struct
       create (fun n -> 
                 let s = String.create n in
                 let r = Unix.read fd s 0 n in
-                s,r)
+                s,r),fd
     with
       | e -> Unix.close fd; raise e
 
@@ -172,3 +180,20 @@ struct
 
 end
 
+
+module Skeleton = 
+struct
+
+  external fishead : Int64.t -> Int64.t -> Int64.t -> Int64.t -> Int32.t -> Stream.packet = "ocaml_ogg_skeleton_fishead"
+
+  let fishead ?(presentation_numerator=Int64.zero) 
+              ?(presentation_denominator=Int64.of_int 1000)
+              ?(basetime_numerator=Int64.zero) 
+              ?(basetime_denominator=Int64.of_int 1000) 
+              ?(utc=Int32.zero) () =
+    fishead presentation_numerator presentation_denominator
+            basetime_numerator basetime_denominator utc
+
+  external eos : unit -> Stream.packet = "ocaml_ogg_skeleton_eos"
+
+end
